#
# Author:: Ranjib Dey (<ranjib@pagerduty.com>)
# Copyright:: Copyright (c) 2014 PagerDuty, Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require 'zk'
require 'zk/locker'
require 'blender/exceptions'

module Blender
  module Lock
    class Zk

      def initialize(name, options = {})
        @path = options[:path] || name
        @hosts = options[:hosts] || 'localhost:2181'
        @chroot = options[:chroot] || :do_nothing
        @thread = options[:thread] || :single
        @timeout = options[:timeout] || 0
      end

      def with_lock
        ZK.open(@hosts) do |zk|
          zk.with_lock(@path, wait: @timeout) do
            yield if block_given?
          end
        end
      rescue ZK::Exceptions::LockWaitTimeoutError => e
        raise LockAcquisitionError, e.message
      end
    end
  end
end
